@@ -3,10 +3,11 @@
 #include <stdlib.h>
 #include <string.h>
 #include <unistd.h>
-#include "context.h"
+#include <wlm/context.h>
 #include <EGL/eglext.h>
-#include "linux-dmabuf-unstable-v1.h"
-#include "mirror-backends.h"
+#include <wlm/mirror-backends.h>
+#include <wlm/util.h>
+#include <wlm/proto/linux-dmabuf-unstable-v1.h>
 
 // --- frame_callback event handlers ---
 
@@ -34,7 +35,7 @@ static void on_frame(
     if (ctx->mirror.backend != NULL) {
         // check if backend failure count exceeded
         if (ctx->mirror.backend->fail_count >= MIRROR_BACKEND_FATAL_FAILCOUNT) {
-            backend_fail(ctx);
+            wlm_mirror_backend_fail(ctx);
         }
 
         if (!ctx->opt.freeze) {
@@ -48,11 +49,11 @@ static void on_frame(
     wl_display_roundtrip(ctx->wl.display);
 
     // render frame, set swap interval to 0 to ensure nonblocking buffer swap
-    draw_texture(ctx);
+    wlm_egl_draw_texture(ctx);
     eglSwapInterval(ctx->egl.display, 0);
     if (eglSwapBuffers(ctx->egl.display, ctx->egl.surface) != EGL_TRUE) {
-        log_error("mirror::on_frame(): failed to swap buffers\n");
-        exit_fail(ctx);
+        wlm_log_error("mirror::on_frame(): failed to swap buffers\n");
+        wlm_exit_fail(ctx);
     }
 
     (void)frame_callback;
@@ -65,7 +66,7 @@ static const struct wl_callback_listener frame_callback_listener = {
 
 // --- init_mirror ---
 
-void init_mirror(ctx_t * ctx) {
+void wlm_mirror_init(ctx_t * ctx) {
     // initialize context structure
     ctx->mirror.current_target = NULL;
     ctx->mirror.frame_callback = NULL;
@@ -78,13 +79,13 @@ void init_mirror(ctx_t * ctx) {
     ctx->mirror.initialized = true;
 
     // finding target output
-    if (!find_output_opt(ctx, &ctx->mirror.current_target, &ctx->mirror.current_region)) {
-        log_error("mirror::init(): failed to find output\n");
-        exit_fail(ctx);
+    if (!wlm_opt_find_output(ctx, &ctx->mirror.current_target, &ctx->mirror.current_region)) {
+        wlm_log_error("mirror::init(): failed to find output\n");
+        wlm_exit_fail(ctx);
     }
 
     // update window title
-    update_title(ctx);
+    wlm_mirror_update_title(ctx);
 
     // add frame callback listener
     ctx->mirror.frame_callback = wl_surface_frame(ctx->wl.surface);
@@ -99,8 +100,8 @@ typedef struct {
 } fallback_backend_t;
 
 static fallback_backend_t auto_fallback_backends[] = {
-    { "dmabuf", init_mirror_dmabuf },
-    { "screencopy", init_mirror_screencopy },
+    { "dmabuf", wlm_mirror_dmabuf_init },
+    { "screencopy", wlm_mirror_screencopy_init },
     { NULL, NULL }
 };
 
@@ -110,14 +111,14 @@ static void auto_backend_fallback(ctx_t * ctx) {
         size_t index = ctx->mirror.auto_backend_index;
         fallback_backend_t * next_backend = &auto_fallback_backends[index];
         if (next_backend->name == NULL) {
-            log_error("mirror::auto_backend_fallback(): no working backend found, exiting\n");
-            exit_fail(ctx);
+            wlm_log_error("mirror::auto_backend_fallback(): no working backend found, exiting\n");
+            wlm_exit_fail(ctx);
         }
 
         if (index > 0) {
-            log_warn("mirror::auto_backend_fallback(): falling back to backend %s\n", next_backend->name);
+            wlm_log_warn("mirror::auto_backend_fallback(): falling back to backend %s\n", next_backend->name);
         } else {
-            log_debug(ctx, "mirror::auto_backend_fallback(): selecting backend %s\n", next_backend->name);
+            wlm_log_debug(ctx, "mirror::auto_backend_fallback(): selecting backend %s\n", next_backend->name);
         }
 
         // uninitialize previous backend
@@ -137,7 +138,7 @@ static void auto_backend_fallback(ctx_t * ctx) {
 
 // --- init_mirror_backend ---
 
-void init_mirror_backend(ctx_t * ctx) {
+void wlm_mirror_backend_init(ctx_t * ctx) {
     if (ctx->mirror.backend != NULL) ctx->mirror.backend->do_cleanup(ctx);
 
     switch (ctx->opt.backend) {
@@ -146,58 +147,135 @@ void init_mirror_backend(ctx_t * ctx) {
             break;
 
         case BACKEND_DMABUF:
-            init_mirror_dmabuf(ctx);
+            wlm_mirror_dmabuf_init(ctx);
             break;
 
         case BACKEND_SCREENCOPY:
-            init_mirror_screencopy(ctx);
+            wlm_mirror_screencopy_init(ctx);
             break;
     }
 
-    if (ctx->mirror.backend == NULL) exit_fail(ctx);
+    if (ctx->mirror.backend == NULL) wlm_exit_fail(ctx);
 }
 
 // --- output_removed ---
 
-void output_removed(ctx_t * ctx, output_list_node_t * node) {
+void wlm_mirror_output_removed(ctx_t * ctx, output_list_node_t * node) {
     if (!ctx->mirror.initialized) return;
     if (ctx->mirror.current_target == NULL) return;
     if (ctx->mirror.current_target != node) return;
 
-    log_error("mirror::output_removed(): output disappeared, closing\n");
-    exit_fail(ctx);
+    wlm_log_error("mirror::output_removed(): output disappeared, closing\n");
+    wlm_exit_fail(ctx);
 }
 
-// --- update_options_mirror ---
+// --- update_title ---
 
-void update_title(ctx_t * ctx) {
-    char * title = NULL;
-    int status = asprintf(&title, "Wayland Output Mirror for %s", ctx->mirror.current_target->name);
-    if (status == -1) {
-        log_error("mirror::update_title(): failed to format window title\n");
-        exit_fail(ctx);
+typedef struct {
+    const char *specifier;
+    char type;
+    union { int d; const char *s; };
+} specifier_t;
+
+static size_t specifier_str(ctx_t * ctx, char *dst, int n, specifier_t specifier) {
+    switch (specifier.type) {
+        case 'd':
+            return snprintf(dst, n, "%d", specifier.d);
+        case 's':
+            return snprintf(dst, n, "%s", specifier.s);
+        default:
+            wlm_log_error("mirror::specifier_str(): unrecognized format type '%d'", specifier.type);
+            wlm_exit_fail(ctx);
+    }
+}
+
+static int format_title(ctx_t * ctx, char ** dst, char * fmt) {
+    specifier_t replacements[] = {
+        {"{x}", 'd', {.d = ctx->mirror.current_region.x}},
+        {"{y}", 'd', {.d = ctx->mirror.current_region.y}},
+        {"{width}", 'd', {.d = ctx->mirror.current_region.width}},
+        {"{height}", 'd', {.d = ctx->mirror.current_region.height}},
+        {"{target_width}", 'd', {.d = ctx->mirror.current_target->width}},
+        {"{target_height}", 'd', {.d = ctx->mirror.current_target->height}},
+        {"{target_output}", 's', {.s = ctx->mirror.current_target->name}}
+    };
+
+    size_t length = strlen(fmt);
+    for (size_t f = 0; f < ARRAY_LENGTH(replacements); f++) {
+        const char *fmt_cursor = fmt;
+        size_t spec_len = strlen(replacements[f].specifier);
+
+        while ((fmt_cursor = strstr(fmt_cursor, replacements[f].specifier))) {
+            length += specifier_str(ctx, NULL, 0, replacements[f]) - spec_len;
+            fmt_cursor += spec_len;
+        }
     }
 
-    set_window_title(ctx, title);
+    *dst = (char *) calloc(++length, sizeof(char));
+    if (*dst == NULL) {
+        wlm_log_error("mirror::format_title(): failed to allocate memory\n");
+        wlm_exit_fail(ctx);
+    }
+
+    char *dst_cursor = *dst;
+    const char *fmt_cursor = fmt;
+    while (*fmt_cursor) {
+        bool replaced = false;
+        for (size_t f = 0; f < ARRAY_LENGTH(replacements); f++) {
+            size_t spec_len = strlen(replacements[f].specifier);
+            if (strncmp(fmt_cursor, replacements[f].specifier, spec_len) == 0) {
+                size_t written = specifier_str(ctx, dst_cursor, length, replacements[f]);
+                dst_cursor += written;
+                fmt_cursor += spec_len;
+                length -= written;
+                replaced = true;
+                break;
+            }
+        }
+        if (!replaced) {
+            *dst_cursor++ = *fmt_cursor++;
+            length--;
+        }
+    }
+    *dst_cursor = '\0';
+
+
+    return 0;
+}
+
+void wlm_mirror_update_title(ctx_t * ctx) {
+    char * title = NULL;
+    char * title_fmt = "Wayland Output Mirror for {target_output}";
+    if (ctx->opt.window_title != NULL) {
+        title_fmt = ctx->opt.window_title;
+    }
+
+    int status = format_title(ctx, &title, title_fmt);
+    if (status == -1) {
+        wlm_log_error("mirror::update_title(): failed to format window title\n");
+        wlm_exit_fail(ctx);
+    }
+
+    wlm_wayland_window_set_title(ctx, title);
     free(title);
 }
 
 // --- backend_fail ---
 
-void backend_fail(ctx_t * ctx) {
+void wlm_mirror_backend_fail(ctx_t * ctx) {
     if (ctx->opt.backend == BACKEND_AUTO) {
         auto_backend_fallback(ctx);
     } else {
-        exit_fail(ctx);
+        wlm_exit_fail(ctx);
     }
 }
 
 // --- cleanup_mirror ---
 
-void cleanup_mirror(ctx_t * ctx) {
+void wlm_mirror_cleanup(ctx_t * ctx) {
     if (!ctx->mirror.initialized) return;
 
-    log_debug(ctx, "mirror::cleanup(): destroying mirror objects\n");
+    wlm_log_debug(ctx, "mirror::cleanup(): destroying mirror objects\n");
 
     if (ctx->mirror.backend != NULL) ctx->mirror.backend->do_cleanup(ctx);
     if (ctx->mirror.frame_callback != NULL) wl_callback_destroy(ctx->mirror.frame_callback);
