@@ -1,10 +1,10 @@
 #include <stdlib.h>
 #include <string.h>
 
-#include "context.h"
-#include "version.h"
+#include <wlm/context.h>
+#include <wlm/version.h>
 
-void init_opt(ctx_t * ctx) {
+void wlm_opt_init(ctx_t * ctx) {
     ctx->opt.verbose = false;
     ctx->opt.stream = false;
     ctx->opt.show_cursor = true;
@@ -19,14 +19,15 @@ void init_opt(ctx_t * ctx) {
     ctx->opt.region = (region_t){ .x = 0, .y = 0, .width = 0, .height = 0 };
     ctx->opt.output = NULL;
     ctx->opt.fullscreen_output = NULL;
+    ctx->opt.window_title = NULL;
 }
 
-void cleanup_opt(ctx_t * ctx) {
+void wlm_cleanup_opt(ctx_t * ctx) {
     if (ctx->opt.output != NULL) free(ctx->opt.output);
     if (ctx->opt.fullscreen_output != NULL) free(ctx ->opt.fullscreen_output);
 }
 
-bool parse_scaling_opt(scale_t * scaling, scale_filter_t * scaling_filter, const char * scaling_arg) {
+bool wlm_opt_parse_scaling(scale_t * scaling, scale_filter_t * scaling_filter, const char * scaling_arg) {
     if (strcmp(scaling_arg, "f") == 0 || strcmp(scaling_arg, "fit") == 0) {
         *scaling = SCALE_FIT;
         return true;
@@ -47,7 +48,7 @@ bool parse_scaling_opt(scale_t * scaling, scale_filter_t * scaling_filter, const
     }
 }
 
-bool parse_backend_opt(backend_t * backend, const char * backend_arg) {
+bool wlm_opt_parse_backend(backend_t * backend, const char * backend_arg) {
     if (strcmp(backend_arg, "auto") == 0) {
         *backend = BACKEND_AUTO;
         return true;
@@ -62,7 +63,7 @@ bool parse_backend_opt(backend_t * backend, const char * backend_arg) {
     }
 }
 
-bool parse_transform_opt(transform_t * transform, const char * transform_arg) {
+bool wlm_opt_parse_transform(transform_t * transform, const char * transform_arg) {
     transform_t local_transform = { .rotation = ROT_NORMAL, .flip_x = false, .flip_y = false };
 
     if (strcmp(transform_arg, "normal") == 0) {
@@ -72,7 +73,7 @@ bool parse_transform_opt(transform_t * transform, const char * transform_arg) {
 
     char * transform_str = strdup(transform_arg);
     if (transform_str == NULL) {
-        log_error("options::parse_transform_option(): failed to allocate copy of transform argument\n");
+        wlm_log_error("options::parse_transform_option(): failed to allocate copy of transform argument\n");
         return false;
     }
 
@@ -81,12 +82,12 @@ bool parse_transform_opt(transform_t * transform, const char * transform_arg) {
     char * transform_spec = strtok(transform_str, "-");
     while (transform_spec != NULL) {
         if (strcmp(transform_spec, "normal") == 0) {
-            log_error("options::parse_transform_option(): %s must be the only transform specifier\n", transform_spec);
+            wlm_log_error("options::parse_transform_option(): %s must be the only transform specifier\n", transform_spec);
             success = false;
             break;
         } else if (strcmp(transform_spec, "flipX") == 0 || strcmp(transform_spec, "flipped") == 0) {
             if (local_transform.flip_x) {
-                log_error("options::parse_transform_option(): duplicate flip specifier %s\n", transform_spec);
+                wlm_log_error("options::parse_transform_option(): duplicate flip specifier %s\n", transform_spec);
                 success = false;
                 break;
             }
@@ -94,7 +95,7 @@ bool parse_transform_opt(transform_t * transform, const char * transform_arg) {
             local_transform.flip_x = true;
         } else if (strcmp(transform_spec, "flipY") == 0) {
             if (local_transform.flip_y) {
-                log_error("options::parse_transform_option(): duplicate flip specifier %s\n", transform_spec);
+                wlm_log_error("options::parse_transform_option(): duplicate flip specifier %s\n", transform_spec);
                 success = false;
                 break;
             }
@@ -102,7 +103,7 @@ bool parse_transform_opt(transform_t * transform, const char * transform_arg) {
             local_transform.flip_y = true;
         } else if (strcmp(transform_spec, "0") == 0 || strcmp(transform_spec, "0cw") == 0 || strcmp(transform_spec, "0ccw") == 0) {
             if (has_rotation) {
-                log_error("options::parse_transform_option(): duplicate rotation specifier %s\n", transform_spec);
+                wlm_log_error("options::parse_transform_option(): duplicate rotation specifier %s\n", transform_spec);
                 success = false;
                 break;
             }
@@ -111,7 +112,7 @@ bool parse_transform_opt(transform_t * transform, const char * transform_arg) {
             local_transform.rotation = ROT_NORMAL;
         } else if (strcmp(transform_spec, "90") == 0 || strcmp(transform_spec, "90cw") == 0 || strcmp(transform_spec, "270ccw") == 0) {
             if (has_rotation) {
-                log_error("options::parse_transform_option(): duplicate rotation specifier %s\n", transform_spec);
+                wlm_log_error("options::parse_transform_option(): duplicate rotation specifier %s\n", transform_spec);
                 success = false;
                 break;
             }
@@ -120,7 +121,7 @@ bool parse_transform_opt(transform_t * transform, const char * transform_arg) {
             local_transform.rotation = ROT_CW_90;
         } else if (strcmp(transform_spec, "180") == 0 || strcmp(transform_spec, "180cw") == 0 || strcmp(transform_spec, "180ccw") == 0) {
             if (has_rotation) {
-                log_error("options::parse_transform_option(): duplicate rotation specifier %s\n", transform_spec);
+                wlm_log_error("options::parse_transform_option(): duplicate rotation specifier %s\n", transform_spec);
                 success = false;
                 break;
             }
@@ -129,7 +130,7 @@ bool parse_transform_opt(transform_t * transform, const char * transform_arg) {
             local_transform.rotation = ROT_CW_180;
         } else if (strcmp(transform_spec, "270") == 0 || strcmp(transform_spec, "270cw") == 0 || strcmp(transform_spec, "90ccw") == 0) {
             if (has_rotation) {
-                log_error("options::parse_transform_option(): duplicate rotation specifier %s\n", transform_spec);
+                wlm_log_error("options::parse_transform_option(): duplicate rotation specifier %s\n", transform_spec);
                 success = false;
                 break;
             }
@@ -137,7 +138,7 @@ bool parse_transform_opt(transform_t * transform, const char * transform_arg) {
             has_rotation = true;
             local_transform.rotation = ROT_CW_270;
         } else {
-            log_error("options::parse_transform_option(): invalid transform specifier %s\n", transform_spec);
+            wlm_log_error("options::parse_transform_option(): invalid transform specifier %s\n", transform_spec);
             success = false;
             break;
         }
@@ -153,12 +154,12 @@ bool parse_transform_opt(transform_t * transform, const char * transform_arg) {
     return success;
 }
 
-bool parse_region_opt(region_t * region, char ** output, const char * region_arg) {
+bool wlm_opt_parse_region(region_t * region, char ** output, const char * region_arg) {
     region_t local_region = { .x = 0, .y = 0, .width = 0, .height = 0 };
 
     char * region_str = strdup(region_arg);
     if (region_str == NULL) {
-        log_error("options::parse_region_option(): failed to allocate copy of region argument\n");
+        wlm_log_error("options::parse_region_option(): failed to allocate copy of region argument\n");
         return false;
     }
 
@@ -167,7 +168,7 @@ bool parse_region_opt(region_t * region, char ** output, const char * region_arg
     char * output_label = strtok(NULL, " ");
 
     if (position == NULL) {
-        log_error("options::parse_region_option(): missing region position\n");
+        wlm_log_error("options::parse_region_option(): missing region position\n");
         free(region_str);
         return false;
     }
@@ -177,15 +178,15 @@ bool parse_region_opt(region_t * region, char ** output, const char * region_arg
     char * rest = strtok(NULL, ",");
 
     if (x == NULL) {
-        log_error("options::parse_region_option(): missing x position\n");
+        wlm_log_error("options::parse_region_option(): missing x position\n");
         free(region_str);
         return false;
     } else if (y == NULL) {
-        log_error("options::parse_region_option(): missing y position\n");
+        wlm_log_error("options::parse_region_option(): missing y position\n");
         free(region_str);
         return false;
     } else if (rest != NULL) {
-        log_error("options::parse_region_option(): unexpected position component %s\n", rest);
+        wlm_log_error("options::parse_region_option(): unexpected position component %s\n", rest);
         free(region_str);
         return false;
     }
@@ -193,7 +194,7 @@ bool parse_region_opt(region_t * region, char ** output, const char * region_arg
     char * end = NULL;
     local_region.x = strtoul(x, &end, 10);
     if (*end != '\0') {
-        log_error("options::parse_region_option(): invalid x position %s\n", x);
+        wlm_log_error("options::parse_region_option(): invalid x position %s\n", x);
         free(region_str);
         return false;
     }
@@ -201,13 +202,13 @@ bool parse_region_opt(region_t * region, char ** output, const char * region_arg
     end = NULL;
     local_region.y = strtoul(y, &end, 10);
     if (*end != '\0') {
-        log_error("options::parse_region_option(): invalid y position %s\n", y);
+        wlm_log_error("options::parse_region_option(): invalid y position %s\n", y);
         free(region_str);
         return false;
     }
 
     if (size == NULL) {
-        log_error("options::parse_region_option(): missing size\n");
+        wlm_log_error("options::parse_region_option(): missing size\n");
         free(region_str);
         return false;
     }
@@ -216,15 +217,15 @@ bool parse_region_opt(region_t * region, char ** output, const char * region_arg
     char * height = strtok(NULL, "x");
     rest = strtok(NULL, "x");
     if (width == NULL) {
-        log_error("options::parse_region_option(): missing width\n");
+        wlm_log_error("options::parse_region_option(): missing width\n");
         free(region_str);
         return false;
     } else if (height == NULL) {
-        log_error("options::parse_region_option(): missing height\n");
+        wlm_log_error("options::parse_region_option(): missing height\n");
         free(region_str);
         return false;
     } else if (rest != NULL) {
-        log_error("options::parse_region_option(): unexpected size component %s\n", rest);
+        wlm_log_error("options::parse_region_option(): unexpected size component %s\n", rest);
         free(region_str);
         return false;
     }
@@ -232,11 +233,11 @@ bool parse_region_opt(region_t * region, char ** output, const char * region_arg
     end = NULL;
     local_region.width = strtoul(width, &end, 10);
     if (*end != '\0') {
-        log_error("options::parse_region_option(): invalid width %s\n", width);
+        wlm_log_error("options::parse_region_option(): invalid width %s\n", width);
         free(region_str);
         return false;
     } else if (local_region.width == 0) {
-        log_error("options::parse_region_option(): invalid width %d\n", local_region.width);
+        wlm_log_error("options::parse_region_option(): invalid width %d\n", local_region.width);
         free(region_str);
         return false;
     }
@@ -244,11 +245,11 @@ bool parse_region_opt(region_t * region, char ** output, const char * region_arg
     end = NULL;
     local_region.height = strtoul(height, &end, 10);
     if (*end != '\0') {
-        log_error("options::parse_region_option(): invalid height %s\n", height);
+        wlm_log_error("options::parse_region_option(): invalid height %s\n", height);
         free(region_str);
         return false;
     } else if (local_region.height == 0) {
-        log_error("options::parse_region_option(): invalid height %d\n", local_region.height);
+        wlm_log_error("options::parse_region_option(): invalid height %d\n", local_region.height);
         free(region_str);
         return false;
     }
@@ -256,7 +257,7 @@ bool parse_region_opt(region_t * region, char ** output, const char * region_arg
     if (output_label != NULL) {
         *output = strdup(output_label);
         if (*output == NULL) {
-            log_error("options::parse_region_option(): failed to allocate copy of output name\n");
+            wlm_log_error("options::parse_region_option(): failed to allocate copy of output name\n");
             free(region_str);
             return false;
         }
@@ -267,13 +268,13 @@ bool parse_region_opt(region_t * region, char ** output, const char * region_arg
     return true;
 }
 
-bool find_output_opt(ctx_t * ctx, output_list_node_t ** output_handle, region_t * region_handle) {
+bool wlm_opt_find_output(ctx_t * ctx, output_list_node_t ** output_handle, region_t * region_handle) {
     char * output_name = ctx->opt.output;
     output_list_node_t * local_output_handle = NULL;
     region_t local_region = (region_t){ .x = 0, .y = 0, .width = 0, .height = 0 };
 
     if (ctx->opt.output != NULL) {
-        log_debug(ctx, "options::find_output(): searching for output by name\n");
+        wlm_log_debug(ctx, "options::find_output(): searching for output by name\n");
         output_list_node_t * cur = ctx->wl.outputs;
         while (cur != NULL) {
             if (cur->name != NULL && strcmp(cur->name, ctx->opt.output) == 0) {
@@ -285,14 +286,14 @@ bool find_output_opt(ctx_t * ctx, output_list_node_t ** output_handle, region_t
             cur = cur->next;
         }
     } else if (ctx->opt.has_region) {
-        log_debug(ctx, "options::find_output(): searching for output by region\n");
+        wlm_log_debug(ctx, "options::find_output(): searching for output by region\n");
         output_list_node_t * cur = ctx->wl.outputs;
         while (cur != NULL) {
             region_t output_region = {
                 .x = cur->x, .y = cur->y,
                 .width = cur->width, .height = cur->height
             };
-            if (region_contains(&ctx->opt.region, &output_region)) {
+            if (wlm_util_region_contains(&ctx->opt.region, &output_region)) {
                 local_output_handle = cur;
                 output_name = cur->name;
                 break;
@@ -303,32 +304,32 @@ bool find_output_opt(ctx_t * ctx, output_list_node_t ** output_handle, region_t
     }
 
     if (local_output_handle == NULL && ctx->opt.output != NULL) {
-        log_error("options::find_output(): output %s not found\n", ctx->opt.output);
+        wlm_log_error("options::find_output(): output %s not found\n", ctx->opt.output);
         return false;
     } else if (local_output_handle == NULL && ctx->opt.has_region) {
-        log_error("options::find_output(): output for region not found\n");
+        wlm_log_error("options::find_output(): output for region not found\n");
         return false;
     } else if (local_output_handle == NULL) {
-        log_error("options::find_output(): no output or region specified\n");
+        wlm_log_error("options::find_output(): no output or region specified\n");
         return false;
     } else {
-        log_debug(ctx, "options::find_output(): found output with name %s\n", output_name);
+        wlm_log_debug(ctx, "options::find_output(): found output with name %s\n", output_name);
     }
 
     if (ctx->opt.has_region) {
-        log_debug(ctx, "options::find_output(): checking if region in output\n");
+        wlm_log_debug(ctx, "options::find_output(): checking if region in output\n");
         region_t output_region = {
             .x = local_output_handle->x, .y = local_output_handle->y,
             .width = local_output_handle->width, .height = local_output_handle->height
         };
-        if (!region_contains(&ctx->opt.region, &output_region)) {
-            log_error("options::find_output(): output does not contain region\n");
+        if (!wlm_util_region_contains(&ctx->opt.region, &output_region)) {
+            wlm_log_error("options::find_output(): output does not contain region\n");
             return false;
         }
 
-        log_debug(ctx, "options::find_output(): clamping region to output bounds\n");
+        wlm_log_debug(ctx, "options::find_output(): clamping region to output bounds\n");
         local_region = ctx->opt.region;
-        region_clamp(&local_region, &output_region);
+        wlm_util_region_clamp(&local_region, &output_region);
     }
 
     *output_handle = local_output_handle;
@@ -336,7 +337,7 @@ bool find_output_opt(ctx_t * ctx, output_list_node_t ** output_handle, region_t
     return true;
 }
 
-void usage_opt(ctx_t * ctx) {
+void wlm_opt_usage(ctx_t * ctx) {
     printf("usage: wl-mirror [options] <output>\n");
     printf("\n");
     printf("options:\n");
@@ -365,6 +366,7 @@ void usage_opt(ctx_t * ctx) {
     printf("  -r R, --region R              capture custom region R\n");
     printf("        --no-region             capture the entire output (default)\n");
     printf("  -S,   --stream                accept a stream of additional options on stdin\n");
+    printf("        --title N               specify a custom title N for the mirror window\n");
     printf("\n");
     printf("backends:\n");
     printf("  - auto        automatically try the backends in order and use the first that works (default)\n");
@@ -395,17 +397,28 @@ void usage_opt(ctx_t * ctx) {
     printf("    quoted or fully unquoted\n");
     printf("  - unquoted arguments are split on whitespace\n");
     printf("  - no escape sequences are implemented\n");
-    cleanup(ctx);
+    printf("\n");
+    printf("title placeholders:\n");
+    printf("  the title string supports the following placeholders:\n");
+    printf("  - {width}, {height}:               size of the mirrored area\n");
+    printf("  - {x}, {y}:                        offsets on the screen\n");
+    printf("  - {target_width}, {target_height}\n");
+    printf("    {target_output}:                 info about the mirrored device\n");
+    printf("  a few perhaps useful examples:\n");
+    printf("    --title='Wayland Mirror Output {target_output}'\n");
+    printf("    --title='{target_output}:{width}x{height}+{x}+{y}'\n");
+    printf("    --title='resize set {width} {height} move position {x} {y}'\n");
+    wlm_cleanup(ctx);
     exit(0);
 }
 
-void version_opt(ctx_t * ctx) {
-    printf("wl-mirror %s\n", VERSION);
-    cleanup(ctx);
+void wlm_opt_version(ctx_t * ctx) {
+    printf("wl-mirror %s\n", WLM_VERSION);
+    wlm_cleanup(ctx);
     exit(0);
 }
 
-void parse_opt(ctx_t * ctx, int argc, char ** argv) {
+void wlm_opt_parse(ctx_t * ctx, int argc, char ** argv) {
     bool is_cli_args = !ctx->opt.stream;
     bool was_frozen = ctx->opt.freeze;
     bool was_fullscreen = ctx->opt.fullscreen;
@@ -418,9 +431,9 @@ void parse_opt(ctx_t * ctx, int argc, char ** argv) {
 
     while (argc > 0 && argv[0][0] == '-') {
         if (is_cli_args && (strcmp(argv[0], "-h") == 0 || strcmp(argv[0], "--help") == 0)) {
-            usage_opt(ctx);
+            wlm_opt_usage(ctx);
         } else if (strcmp(argv[0], "-V") == 0 || strcmp(argv[0], "--version") == 0) {
-            version_opt(ctx);
+            wlm_opt_version(ctx);
         } else if (strcmp(argv[0], "-v") == 0 || strcmp(argv[0], "--verbose") == 0) {
             ctx->opt.verbose = true;
         } else if (strcmp(argv[0], "--no-verbose") == 0) {
@@ -430,7 +443,7 @@ void parse_opt(ctx_t * ctx, int argc, char ** argv) {
         } else if (strcmp(argv[0], "--no-show-cursor") == 0) {
             ctx->opt.show_cursor = false;
         } else if (strcmp(argv[0], "-n") == 0) {
-            log_warn("options::parse(): -n is deprecated, use --no-show-cursor\n");
+            wlm_log_warn("options::parse(): -n is deprecated, use --no-show-cursor\n");
             ctx->opt.show_cursor = false;
         } else if (strcmp(argv[0], "-i") == 0 || strcmp(argv[0], "--invert-colors") == 0) {
             ctx->opt.invert_colors = true;
@@ -448,8 +461,8 @@ void parse_opt(ctx_t * ctx, int argc, char ** argv) {
             ctx->opt.fullscreen = false;
         } else if (strcmp(argv[0], "--fullscreen-output") == 0) {
             if (argc < 2) {
-                log_error("options::parse(): option %s requires an argument\n", argv[0]);
-                if (is_cli_args) exit_fail(ctx);
+                wlm_log_error("options::parse(): option %s requires an argument\n", argv[0]);
+                if (is_cli_args) wlm_exit_fail(ctx);
             } else {
                 free(ctx->opt.fullscreen_output);
                 ctx->opt.fullscreen = true;
@@ -464,12 +477,12 @@ void parse_opt(ctx_t * ctx, int argc, char ** argv) {
             new_fullscreen_output = true;
         } else if (strcmp(argv[0], "-s") == 0 || strcmp(argv[0], "--scaling") == 0) {
             if (argc < 2) {
-                log_error("options::parse(): option %s requires an argument\n", argv[0]);
-                if (is_cli_args) exit_fail(ctx);
+                wlm_log_error("options::parse(): option %s requires an argument\n", argv[0]);
+                if (is_cli_args) wlm_exit_fail(ctx);
             } else {
-                if (!parse_scaling_opt(&ctx->opt.scaling, &ctx->opt.scaling_filter, argv[1])) {
-                    log_error("options::parse(): invalid scaling mode %s\n", argv[1]);
-                    if (is_cli_args) exit_fail(ctx);
+                if (!wlm_opt_parse_scaling(&ctx->opt.scaling, &ctx->opt.scaling_filter, argv[1])) {
+                    wlm_log_error("options::parse(): invalid scaling mode %s\n", argv[1]);
+                    if (is_cli_args) wlm_exit_fail(ctx);
                 }
 
                 argv++;
@@ -477,12 +490,12 @@ void parse_opt(ctx_t * ctx, int argc, char ** argv) {
             }
         } else if (strcmp(argv[0], "-b") == 0 || strcmp(argv[0], "--backend") == 0) {
             if (argc < 2) {
-                log_error("options::parse(): option %s requires an argument\n", argv[0]);
-                if (is_cli_args) exit_fail(ctx);
+                wlm_log_error("options::parse(): option %s requires an argument\n", argv[0]);
+                if (is_cli_args) wlm_exit_fail(ctx);
             } else {
-                if (!parse_backend_opt(&ctx->opt.backend, argv[1])) {
-                    log_error("options::parse(): invalid backend %s\n", argv[1]);
-                    if (is_cli_args) exit_fail(ctx);
+                if (!wlm_opt_parse_backend(&ctx->opt.backend, argv[1])) {
+                    wlm_log_error("options::parse(): invalid backend %s\n", argv[1]);
+                    if (is_cli_args) wlm_exit_fail(ctx);
                 }
 
                 new_backend = true;
@@ -491,12 +504,12 @@ void parse_opt(ctx_t * ctx, int argc, char ** argv) {
             }
         } else if (strcmp(argv[0], "-t") == 0 || strcmp(argv[0], "--transform") == 0) {
             if (argc < 2) {
-                log_error("options::parse(): option %s requires an argument\n", argv[0]);
-                if (is_cli_args) exit_fail(ctx);
+                wlm_log_error("options::parse(): option %s requires an argument\n", argv[0]);
+                if (is_cli_args) wlm_exit_fail(ctx);
             } else {
-                if (!parse_transform_opt(&ctx->opt.transform, argv[1])) {
-                    log_error("options::parse(): invalid transform %s\n", argv[1]);
-                    if (is_cli_args) exit_fail(ctx);
+                if (!wlm_opt_parse_transform(&ctx->opt.transform, argv[1])) {
+                    wlm_log_error("options::parse(): invalid transform %s\n", argv[1]);
+                    if (is_cli_args) wlm_exit_fail(ctx);
                 }
 
                 argv++;
@@ -504,13 +517,13 @@ void parse_opt(ctx_t * ctx, int argc, char ** argv) {
             }
         } else if (strcmp(argv[0], "-r") == 0 || strcmp(argv[0], "--region") == 0) {
             if (argc < 2) {
-                log_error("options::parse(): option %s requires an argument\n", argv[0]);
-                if (is_cli_args) exit_fail(ctx);
+                wlm_log_error("options::parse(): option %s requires an argument\n", argv[0]);
+                if (is_cli_args) wlm_exit_fail(ctx);
             } else {
                 char * new_region_output = NULL;
-                if (!parse_region_opt(&ctx->opt.region, &new_region_output, argv[1])) {
-                    log_error("options::parse(): invalid region %s\n", argv[1]);
-                    if (is_cli_args) exit_fail(ctx);
+                if (!wlm_opt_parse_region(&ctx->opt.region, &new_region_output, argv[1])) {
+                    wlm_log_error("options::parse(): invalid region %s\n", argv[1]);
+                    if (is_cli_args) wlm_exit_fail(ctx);
                 } else {
                     ctx->opt.has_region = true;
                     free(region_output);
@@ -526,13 +539,28 @@ void parse_opt(ctx_t * ctx, int argc, char ** argv) {
             ctx->opt.region = (region_t){ .x = 0, .y = 0, .width = 0, .height = 0 };
         } else if (strcmp(argv[0], "-S") == 0 || strcmp(argv[0], "--stream") == 0) {
             ctx->opt.stream = true;
+        } else if (strcmp(argv[0], "--title") == 0) {
+            if (argc < 2) {
+                wlm_log_error("options::parse(): option %s requires an argument\n", argv[0]);
+                if (is_cli_args) wlm_exit_fail(ctx);
+            } else {
+                if (strlen(argv[1]) <= 0) {
+                    wlm_log_error("options::parse(): invalid empty title\n");
+                    if (is_cli_args) wlm_exit_fail(ctx);
+                } else {
+                    ctx->opt.window_title = argv[1];
+                }
+
+                argv++;
+                argc--;
+            }
         } else if (strcmp(argv[0], "--") == 0) {
             argv++;
             argc--;
             break;
         } else {
-            log_error("options::parse(): invalid option %s\n", argv[0]);
-            if (is_cli_args) exit_fail(ctx);
+            wlm_log_error("options::parse(): invalid option %s\n", argv[0]);
+            if (is_cli_args) wlm_exit_fail(ctx);
         }
 
         argv++;
@@ -542,8 +570,8 @@ void parse_opt(ctx_t * ctx, int argc, char ** argv) {
     if (argc > 0) {
         arg_output = strdup(argv[0]);
         if (arg_output == NULL) {
-            log_error("options::parse(): failed to allocate copy of output name\n");
-            if (is_cli_args) exit_fail(ctx);
+            wlm_log_error("options::parse(): failed to allocate copy of output name\n");
+            if (is_cli_args) wlm_exit_fail(ctx);
         } else {
             new_output = true;
         }
@@ -574,8 +602,8 @@ void parse_opt(ctx_t * ctx, int argc, char ** argv) {
         // must be the same
         // region must be in this output
         if (strcmp(region_output, arg_output) != 0) {
-            log_error("options::parse(): region and argument output differ: %s vs %s\n", region_output, arg_output);
-            if (is_cli_args) exit_fail(ctx);
+            wlm_log_error("options::parse(): region and argument output differ: %s vs %s\n", region_output, arg_output);
+            if (is_cli_args) wlm_exit_fail(ctx);
         }
         ctx->opt.output = region_output;
     } else if (new_output && !new_region) {
@@ -583,46 +611,46 @@ void parse_opt(ctx_t * ctx, int argc, char ** argv) {
         ctx->opt.output = arg_output;
     } else if (!new_output && !new_region && is_cli_args) {
         // no output or region specified
-        usage_opt(ctx);
+        wlm_opt_usage(ctx);
     }
 
     if (
         ctx->opt.output != NULL && ctx->opt.fullscreen_output != NULL &&
         strcmp(ctx->opt.output, ctx->opt.fullscreen_output) == 0
     ) {
-        log_error("options::parse(): fullscreen_output cannot be same as the output to be mirrored");
-        exit_fail(ctx);
+        wlm_log_error("options::parse(): fullscreen_output cannot be same as the output to be mirrored");
+        wlm_exit_fail(ctx);
     }
 
     if (argc > 1) {
-        log_error("options::parse(): unexpected trailing arguments after output name\n");
-        if (is_cli_args) exit_fail(ctx);
+        wlm_log_error("options::parse(): unexpected trailing arguments after output name\n");
+        if (is_cli_args) wlm_exit_fail(ctx);
     }
 
     if (!is_cli_args && ctx->opt.fullscreen && (!was_fullscreen || new_fullscreen_output)) {
-        set_window_fullscreen(ctx);
+        wlm_wayland_window_set_fullscreen(ctx);
     } else if (!is_cli_args && !ctx->opt.fullscreen && was_fullscreen) {
-        unset_window_fullscreen(ctx);
+        wlm_wayland_window_unset_fullscreen(ctx);
     }
 
     output_list_node_t * target_output = NULL;
     region_t target_region = (region_t){ .x = 0, .y = 0, .width = 0, .height = 0 };
-    if (!is_cli_args && find_output_opt(ctx, &target_output, &target_region)) {
+    if (!is_cli_args && wlm_opt_find_output(ctx, &target_output, &target_region)) {
         ctx->mirror.current_target = target_output;
         ctx->mirror.current_region = target_region;
     }
 
     if (!is_cli_args && new_backend) {
-        init_mirror_backend(ctx);
+        wlm_mirror_backend_init(ctx);
     }
 
     if (!is_cli_args && !was_frozen && ctx->opt.freeze) {
-        freeze_framebuffer(ctx);
+        wlm_egl_freeze_framebuffer(ctx);
     }
 
     if (!is_cli_args) {
-        update_uniforms(ctx);
-        update_title(ctx);
+        wlm_egl_update_uniforms(ctx);
+        wlm_mirror_update_title(ctx);
     }
 }
 
